/*
JMP Clinical Analytical Process

Name: Progression Free Survival Plots
Authors: saslii, SAS Institute Inc.
History:
   saslii 18Aug17 Initial Coding

Description: 
      This process creates waterfall plots to show the distribution of changes in test measurements for a given Findings domain
      across subjects (ordered by their magnitude of change). The Findings measurements are summarized based on a specified summary 
      statistic and a waterfall plot is created for each Findings test.
*/

/*
-----------------------------------------------------------
Copyright (C) 2010 SAS Institute, Inc. All rights reserved. 

Notice: 
The above copyright notice and this notice must appear in
any whole or partial copy of this code and any related
documentation. Permission to use, copy, and modify the
source code contained in this file is hereby granted, but
is limited to customers of SAS with a valid license for the
SAS software product with which this file was provided. 
Except as expressly set forth herein, the terms of such
license apply.
-----------------------------------------------------------
*/

*/
*******************************************************************;
*     Clinical Variable Specification Block of CDISC Variable Usage;
*******************************************************************;
/*    
$CVS_START$
$INCF: {"LifeSciences\MacroLib\CDISCParms.sas","LifeSciences\MacroLib\FindingsMacros.sas"}$
$INCB: {{"LoadData","LoadDataForDM","LoadDataO"},{"GeneralOptionsOncology"}}$
$INCD: {{"RS","DM","DS"},{"RS"}}$
$CMP: RSYS$ $UE: RS.RSTEST$ $NOTES: To get overall response information in RS to represent Response Duration.$
$CMP: RSYS$ $UE: RS.RSSTRESC and (RS.RSDY or RS.RSDTC or ((SV.VISITNUM or SV.VISIT) and (SV.SVSTDY or (SV.SVSTDTC and DM.RFSTDTC))))$ 
 $NOTES: Used for capturing progressive disease values and dates, events, and time to events.$
$CMP: CPV.{"End Date Variable to Use for Censoring","Subject Reference End Date"}$ $UE: DM.RFSTDTC and DM.RFENDTC$ $NOTES: To determine the time-to-event variable for performing survival analysis.$
$CMP: CPV.{"End Date Variable to Use for Censoring","Date of Last Study Treatment"}$ $UE: DM.RFSTDTC and DM.RFXENDTC$ $NOTES: To determine the time-to-event variable for performing survival analysis.$
$CMP: CPV.{"End Date Variable to Use for Censoring","Date of End of Participation"}$ $UE: DM.RFSTDTC and DM.RFPENDTC$ $NOTES: To determine the time-to-event variable for performing survival analysis.$
$CMP: CPV.{"End Date Variable to Use for Censoring","Last Response Assessment Date"}$ $UE: DM.RFSTDTC and RS.RSDTC$ $NOTES: To determine the time-to-event variable for performing survival analysis.$
$CVS_STOP$
*/

*ProcessVariables;

%global StudyName InPath AInPath FindDTests TrtUse TrtVar WhereTests WhereForReports RMUnscheduledVisits
   IncSF PopulationSet MeasurementValues Statistic  NormRef dieThreshold shadedCI CensorDateVar AnalysisType
   ScreenControlValue AgeGroupFlag AgeGroup1 AgeGroup2 AgeGroup3 AgeGroup4 AgeGroup5 
   DesignByVars SubjectFilter WhereStmt  MERGESUPPDOM TimeScale BaselineTimeWindow BaselineCalc LogShft NormRef
   DisplaySubjectAxis OverlayTrtGroups ShowStat OutPath ServerOutPath MacroPath ProgressionSurPlots_OutData 
   ProgressionPlotsScript FDeltaDrop objReferenceName  BarDateVar CensorMark;

*ProcessBody;

%macro ProgressionFreeSurvivalPlots;
*options mprint;
%let exerr = 0;
*initialization and argument checking;
%let ProcessName = ProgressionFreeSurvivalPlots; 
%global ADSLBLIND DMBlind DMXtra;
%let DMBlind=;
%let ADSLBLIND=; 
%let DMXtra =;

%include "&MacroPath./UtilityPathMacros.sas" / nosource;
%include "&MacroPath./UtilityMacros.sas" / nosource;
%include "&MacroPath./CDISCParms.sas";
%include "&MacroPath./FindingsMacros.sas" /nosource;

*** Prepare ADSL ***;
%PREPADSL(_ConvertEndDateFlag=0);
%let WhereForReports=%nrbquote(&WhereStmt);
%if &exiterror %then %goto exit;

* RS domain is required to run this report;
%if ^%sysfunc(findw(&domains, RS)) %then %do;
   %put ERROR: The RS or ADRS data set is required for progression free survival analysis.;
   %let exiterror = 1;
   %goto exit;
%end;

* Check age grouping;
%if %upcase(&AgeGroupFlag)=YES %then %do;
	%let Aflag = 0;
   %if %length(%trim(&AgeGroup1)) %then %let FAG1 = 1;
   %else %let FAG1 = 0;
   %if %length(%trim(&AgeGroup2)) %then %let FAG2 = 1;
   %else %let FAG2 = 0;
   %if %length(%trim(&AgeGroup3)) %then %let FAG3 = 1;
   %else %let FAG3 = 0;
   %if %length(%trim(&AgeGroup4)) %then %let FAG4 = 1;
   %else %let FAG4 = 0;
   %if %length(%trim(&AgeGroup5)) %then %let FAG5 = 1;
   %else %let FAG5 = 0;
   %if %eval(&FAG1+&FAG2+&FAG3+&FAG4+&FAG5) < 2 %then %do;
      %ERROR: You must specify at least 2 Age groups.;
      %let exiterror = 1;
      %goto exit;
   %end;
   %let Aflag = %sysevalf(&FAG1+&FAG2+&FAG3+&FAG4+&FAG5);
%end;
%else %do;
   %let AgeGroup1 = %nrbquote(%LocaleGet(All_Text));
   %let FAG1 = 1;
   %let FAG2 = 0;
   %let FAG3 = 0;
   %let FAG4 = 0;
   %let FAG5 = 0;
%end;
%if &CensorDateVar = RFENDTC %then %do;
	%let CensorDateValue = %nrbquote(%LocaleGet(SubjectReferenceEndDate_Text));
%end;
%else %if &CensorDateVar = RFXENDTC %then %do;
	%if %sysfunc(findw(%upcase(&ADSLVARS), TRTEDT)) %then %let CensorDateVar = TRTEDT;
	%let CensorDateValue = %nrbquote(%LocaleGet(DateofLastStudyTreatment_Text));
%end;
%else %if &CensorDateVar = RFPENDTC %then %do;
	%if %sysfunc(findw(%upcase(&ADSLVARS), EOSDT)) %then %let CensorDateVar = EOSDT;
	%let CensorDateValue = %nrbquote(%LocaleGet(DateofEndofParticipation_Text));
%end;
%else %do;
	%let CensorDateValue = %nrbquote(%LocaleGet(LastResponseAssessmentDate_Text));
%end;

*** check if there is any overlapping between any age groups ***;
%if %upcase(&AgeGroupFlag)=YES %then %do;
	%do i =1 %to &Aflag;
		data age_sub&i (keep=USUBJID);
			set &in_adsl;
			if &&AgeGroup&i;
		run;
		proc sort data =age_sub&i;
			by USUBJID;
		run;
	%end;
	%let AflagL=%eval(&Aflag+1);
	%do i =1 %to &AflagL;
		%do j=%eval(&i+1) %to &Aflag;
			data suberr;
				merge age_sub&i(in=ina) age_sub&j(in=inb);
				by USUBJID;
				if ina & inb;
			run;
			%NObsVars(suberr);
			%put suberr=&nobs;
			%if &nobs %then %do;
				%put ERROR: There is an overlap between the age groups: &&AgeGroup&i and &&AgeGroup&j.;
				%put ERROR: Please redefine the age groups such that they are mutually exclusive with no overlap.;
				%let exiterror = 1;
   			%goto exit;
			%end;
		%end;
	%end;
%end;

*create death label from DS and then merge with adsl;
%let trRace = %nrbquote(%LocaleGet(Race_Text));
%let trSex = %nrbquote(%LocaleGet(Sex_Text));
%let trDemographics = %nrbquote(%LocaleGet(Demographics_Text));
%let DeathLabel = %nrbquote(%LocaleGet(SubjectDied_Text));
%let trdeath = %upcase(%nrbquote(%LocaleGet(Death_Text)));
%let trdied = %upcase(%nrbquote(%LocaleGet(Died_Text)));
%let trdead = %upcase(%nrbquote(%LocaleGet(Dead_Text)));
%let trDispositionEvent = %upcase(%nrbquote(%LocaleGet(DispositionEvent_Text)));
%let trEndoftreatment = %upcase(%nrbquote(%LocaleGet(EndofTreatment_Text)));
%let trEndofstudy = %upcase(%nrbquote(%LocaleGet(EndofStudy_Text)));
%let trTimetoprogression = %upcase(%nrbquote(%LocaleGet(TimetoProgression_Text)));
%let _DM_Flag_ = 1;
%let doStudy = 1;
%let doAge = 1;
%let doSex = 1;
%let doRace = 1;
%let doSite = 1;
%let doTrt = 1;
%let stageFlag =0;
%let siflg = %sysfunc(indexw(&ADSLVARS, STUDYID, ' '));
%let aflg = %sysfunc(indexw(&ADSLVARS, AGE, ' '));
%let rflg = %sysfunc(indexw(&ADSLVARS, RACE, ' '));
%let sflg = %sysfunc(indexw(&ADSLVARS, SITEID, ' '));
%let seflg = %sysfunc(indexw(&ADSLVARS, SEX, ' '));
%let Rsscatflg = %sysfunc(indexw(&RSVARS, RSSCAT, ' '));
%let visitflg = %sysfunc(indexw(&RSVARS, VISIT, ' '));
%let visitnumflg = %sysfunc(indexw(&RSVARS, VISITNUM, ' '));
%let dthdtcflag = %sysfunc(indexw(&ADSLVARS, DTHDTC, ' '));
%let dsFlag = %sysfunc(indexw(&domains, DS, ' '));

%if ^&siflg %then %do;
   %let doStudy = 0;
   %let Study = ;
%end;
%if ^&aflg %then %do;
   %let doAge = 0;
   %let Age = ;
%end;
%if ^&rflg %then %do;
   %let doRace = 0;
   %let Race = ;
%end;
%if ^&sflg %then %do;
   %let doSite = 0;
   %let Site = ;
%end;
%if ^&seflg %then %do;
   %let doSex = 0;
   %let Sex = ;
%end;
%if %UPCASE(&TimeScale)=DAYS %then %do;
	%let TimeScaleVa= 1;
	%let trTimeScale = %nrbquote(%LocaleGet(Days_Text));
%end;
%else %if %UPCASE(&TimeScale)=WEEKS %then %do;
	%let TimeScaleVa= 7;
	%let trTimeScale = %nrbquote(%LocaleGet(Weeks_Text));
%end;
%else %do;
	%let TimeScaleVa= 30;
	%let trTimeScale = %nrbquote(%LocaleGet(Months_Text));
%end;

%PrepOutDS(ProgressionSurPlots_OutData,OutDataName,rs,_pfs);

data adsl;
	set &in_adsl;
	* if DTHDTC and DTHFL exist in DM table;
	%if &dthdtcflag %then %do;
		%calcStudyDay(DTHDTC,DTHDAYS);
	%end;
	* calculate censor cut off days;
	%if &CensorDateVar = RFENDTC %then %do;
		%calcStudyDay(RFENDTC,DCUTDAYS);
	%end;
	%else	%if &CensorDateVar = RFXENDTC %then %do;
		%calcStudyDay(RFXENDTC,DCUTDAYS);
	%end;
	%else	%if &CensorDateVar = TRTEDT %then %do;
		%calcStudyDay(TRTEDT,DCUTDAYS);
	%end;
	%else	%if &CensorDateVar = RFPENDTC %then %do;
		%calcStudyDay(RFPENDTC,DCUTDAYS);
	%end;
	%else	%if &CensorDateVar = EOSDT %then %do;
		%calcStudyDay(EOSDT,DCUTDAYS);
	%end;
run;

* handeling DS dataset;
* Get death from DECOD and date of death since it is also called progressed;
%if &dsFlag %then %do;
	%LOADDATA(domlist=DS);
	* only keep subjects who died;
	data died;
	   set &in_ds;
  		DSMONTH = STDY/&TimeScaleVa;
  		if kupcase(DSDECOD) in 
  		 (%unquote(%LocaleGetAll(Death_Text, _case = u)),
  		  %unquote(%LocaleGetAll(Died_Text, _case = u)),
  		  %unquote(%LocaleGetAll(Dead_Text, _case = u))) &
  		 ^missing(DSSTDTC);
	   DIED = 1;
	   keep USUBJID DSMONTH STDY DIED;
	run;
	proc sort data =died nodupkey;
	 by USUBJID;
	run;

	proc sql;
	   create table adsl_ds(keep=USUBJID STUDYID DSMONTH %if &dthdtcflag %then DTHDAYS; %if &CensorDateVar ^= RSDTC %then DCUTDAYS; 
	   	STDY DIED &TrtName AGE SEX RACE ETHNIC SITEID COUNTRY) as
	      select *
	      from adsl d1
	      left join died d2
	      on d1.USUBJID=d2.USUBJID;
	quit;
	data adsl_ds;
		set adsl_ds;
		%if &dthdtcflag %then %do;
			if missing(DIED) & ^missing(DTHDAYS) then do;
				DSMONTH= DTHDAYS/&TimeScaleVa;
				DIED =1;
			end;
		%end;
	run;
%end;
%else %do;
	data adsl_ds(keep=USUBJID STUDYID DCUTDAYS DTHDAYS DSMONTH DIED &TrtName AGE SEX RACE ETHNIC SITEID COUNTRY);
		set adsl;
		%if &dthdtcflag %then %do;
		  	DSMONTH = DTHDAYS/&TimeScaleVa;
			if ^missing(DTHDAYS) then DIED =1;
		%end;
	run;
%end;

proc sort data=adsl_ds out = trtvar(keep=USUBJID &TrtName) nodupkey;
   by &TrtName;
run;

%let RSDTCflag =0;
%if &CensorDateVar =RSDTC %then %do;
	%let RSDTCflag =1;
	%if (^%index(&DMVars,RFSTDTC) and ^%index(&ADSLVars,TRTSDT)) | ^%index(&RSVars,&CensorDateVar) %then %do;
	   %put [MS_S_0145];
	   %put ERROR: Either RFSTDTC and TRTSDT are missing or &CensorDateVar is missing. These variables are required to determine the time-to-event variable for performing survival analysis.; 
	   %let exiterror= 1;
	   %goto exit;
	%end;
%end;
%else %do;
	%if (^%index(&DMVars,RFSTDTC) and ^%index(&ADSLVars,TRTSDT)) | ^%index(&DMVars,&CensorDateVar) %then %do;
	   %put [MS_S_0145];
	   %put ERROR: Either RFSTDTC and TRTSDT are missing or &CensorDateVar is missing. These variables are required to determine the time-to-event variable for performing survival analysis.; 
	   %let exiterror= 1;
	   %goto exit;
	%end;
%end;

%LOADDATA(domlist=RS);
*transpose rs by rsorres;
%let RSDayflag =%sysfunc(indexw(&RSVARS, RSDY, ' '));
%let Epochflg = %sysfunc(indexw(&RSVARS, EPOCH, ' '));

* preprocessing rs data to remove unscheduled if specified;
data rs;
	set rs;
	if kupcase(RSTEST) in (%unquote(%LocaleGetAll(OverallResponse_Text, _case = u)));
	if %unquote(%LocaleGetAll(Not_Text, _case = u, _indexv = RSSTRESC, _indexvamp = 0)) or 
	   %unquote(%LocaleGetAll(Non_Text, _case = u, _indexv = RSSTRESC, _indexvamp = 0)) or
	   %unquote(%LocaleGetAll(Relapse_Text, _case = u, _indexv = RSSTRESC, _indexvamp = 0)) or
	   kupcase(RSSTRESC)="NE" or kupcase(RSSTRESC)="" then delete;
	%if &RMUnscheduledVisits =Yes %then %do;
		if kupcase(RSTPT) in (%unquote(%LocaleGetAll(UnscheduledVisit_Text, _case = u))) then delete;
	%end;
run;
%Nobsvars(rs);
%if &nobs = 0 %then %do;
	%put ERROR: No observations had an RSTEST value of 'Overall Response' without an RSSTRESC value substring of 'NE', 'Non', 'Not', or 'Relapse'.;
	%let exiterror = 0;
	%goto exit;
%end;

* check if RSDY or RS or RSDTC is missing, if so then check SV to use visit date in SV domainvisit ;
* if neither of the date is available, throw an error messge;
data rs_missing;
	set rs;
	if RSDTC="" and RSDY = 0 then delete;
run;
%Nobsvars(rs_missing);
%let dateerror =0;
%let rsvisit=;
%if &nobs = 0 %then %do;
	* if date in RS domain is not available or missing, we can use the date in sv domain;
	%if %sysfunc(findw(&domains, SV)) %then %do;
			%if %sysfunc(findw(%upcase(&SVVARS), VISITNUM)) & &visitnumflg %then %let VISITNVAL = VISITNUM;
			%else %if %sysfunc(findw(%upcase(&SVVARS), VISIT)) & &visitflg %then %let VISITNVAL = VISIT;
			%else %let VISITNVAL = ;
			%if %length(&VISITNVAL)  %then %do;
				%LOADDATA(domlist=SV);
				proc sort data=sv(keep=usubjid &VISITNVAL stdy);
					by USUBJID &VISITNVAL;
				run;
				proc sort data =rs(drop=STDY);
					by USUBJID &VISITNVAL;
				run;
				data rs;
					merge rs(in=ina) sv(in=inb);
					by USUBJID &VISITNVAL;
					if ina;
				run;
				data rs;
					set rs;
					if missing(STDY) then delete;
				run;				
				%Nobsvars(rs);
				%if &nobs = 0 %then %let dateerror =1;
			%end;
			%else %let dateerror =1;
	%end;
	%else %let dateerror =1;
%end;
%if &dateerror %then %do;
	%put ERROR: Analysis cannot be performed due to missing required variable for overall responses: response dates (RSDY or RSDTC) are missing/null in the RS domain.;
	%put ERROR: At least one of them is required for Progression Free Survival Plot.;
	%let exiterror = 0;
	%goto exit;
%end;

* if censor cutoff date is the last response assessment date, need to get the last RSDTC;
%if &CensorDateVar = RSDTC %then %do;
	* remove subjects with RS days <=1;
	data rs;
		set rs;
		if STDY >1;
	run;
	proc sort data =rs;
		by USUBJID descending STDY;
	run;

	data rs;
		retain DCUTDAYS 0;
		set rs;
		by USUBJID;
		if first.USUBJID then DCUTDAYS=STDY;
		else DCUTDAYS=DCUTDAYS;		
	run;
%end;

* for subjects with progressive disease, get the first response time;
data progressed;
	set rs;
	RSMONTH = STDY/&TimeScaleVa;
	if %unquote(%LocaleGetAll(ProgressiveDisease_Text, _case = u, _indexv = RSSTRESC, _indexvamp = 0)) or kindex(kupcase(RSSTRESC),"PD") then progressed =1;
	else progressed =0;
	keep USUBJID RSMONTH progressed RSSTRESC %if &CensorDateVar = RSDTC %then DCUTDAYS;;
	label DCUTDAYS="%nrbquote(%LocaleGet(CensorCutoffDays_Text))";
run;
proc sort data=progressed;
	by USUBJID descending progressed RSMONTH;
run;
data progressed;
	set progressed;
	by USUBJID;
	if first.USUBJID;
run;
*merge RS with main dataset;
proc sql noprint;
   create table adsl_rs as
   select * from adsl_ds as a left join progressed as b 
   on a.USUBJID = b.USUBJID;
quit;

*get number of subjects who didnt die and who are missing the censor date value and delete them;
%let DCUTDmissflag = 0;
data DCUTDaymissing;
	set adsl_rs;
	if missing(DCUTDAYS);
run;
%nobsvars(DCUTDaymissing);
%if &nObs > 0 %then %do;
	%let DCUTDmissflag =1;
	%let missingObs= &nObs;
%end;

* get time to progression days;
data adsl_rs;
	set adsl_rs;
	
	%if %upcase(&AnalysisType) = TTP %then %do;
		if missing (DCUTDAYS) and missing(DSMONTH) then delete;
		DCUTDAYS = DCUTDAYS/&TimeScaleVa + &dieThreshold;
		DCUTDAYS = min(DCUTDAYS,DSMONTH);
		if progressed = 1 then ResponseTime = RSMONTH;
		else ResponseTime = DCUTDAYS;
	%end;
	
	%else %do;
		if missing (DCUTDAYS) then delete;
		DCUTDAYS = DCUTDAYS/&TimeScaleVa + &dieThreshold;
		if progressed = 1 then ResponseTime = RSMONTH;
		else do;
			if DIED =1 then ResponseTime=DSMONTH;
			else ResponseTime = DCUTDAYS;
		end;
	%end;
run;

* compre time to progression with censor cut off date;
* if time to progression is within cut off date, then set censor to 0;
* if time to progression is out of cut off date, then set censor to 1;
data rs_svFinal;
	set adsl_rs;
	%if %upcase(&AnalysisType) = TTP %then %do;
		if progressed =1 then do;
			if ResponseTime > DCUTDAYS then do;
				CENSOR =1;
				ResponseTime=DCUTDAYS;
			end;
			else CENSOR = 0;
		end;
		else CENSOR=1;
	%end;
	%else %do;
		if DIED =1 or progressed =1 then do;
			if ResponseTime > DCUTDAYS then do;
				CENSOR =1;
				ResponseTime=DCUTDAYS;
			end;
			else CENSOR = 0;
		end;
		else CENSOR=1;
	%end;
	label ResponseTime="%nrbquote(%LocaleFill(%nrbquote(%LocaleGet(XtoProgression_Text)), %nrbquote(&trTimeScale)))"; /*"&trTimetoprogression";*/
run;
%if &exiterror %then %goto exit;

%let USUBJIDLabel = USUBJID;
%CheckLabel(rs_svFinal,USUBJIDLabel);
%let StudyLabel = STUDYID;
%CheckLabel(rs_svFinal,StudyLabel);
%let AgeLabel = AGE;
%CheckLabel(rs_svFinal,AgeLabel);
%let SexLabel = SEX;
%CheckLabel(rs_svFinal,SexLabel);
%let RACELabel = RACE;
%CheckLabel(rs_svFinal,RACELabel);
%let SiteLabel = SITEID;
%CheckLabel(rs_svFinal,SiteLabel);
%let ARMLabel = &TrtName;
%CheckLabel(rs_svFinal,ARMLabel);
%let ProgressionLabel=ResponseTime;
%CheckLabel(rs_svFinal,ProgressionLabel);
%let Treatment=treatment;
%put TrtLabel =&TrtLabel &ARMlabel;

%if %UPCASE(&TimeScale)=DAYS %then %do;
   %let TimeVar = STDY;
   %let STvar = STDY;
   %let ENvar = STDY;
%end;
%else %do;
   %let TimeVar = STWK;
   %let STvar = STWK;
   %let ENvar = STWK;
%end;

* create JSL file;
%PrepOutFile(ProgressionPlotsScript,ProgressionPlotsScriptName,&StudyName &ProcessName Script for .jsl);
data outlib.&OutDataName;
   set rs_svFinal;
   * added Age Groups;
   %if %upcase(&AgeGroupFlag)=YES %then %do;
      %if &FAG1 %then %do;
         if (&AgeGroup1) then NewAgeGroup = 1; 
      %end;
      %if &FAG2 %then %do;
         if (&AgeGroup2) then NewAgeGroup = 2; 
      %end;
      %if &FAG3 %then %do;
         if (&AgeGroup3) then NewAgeGroup = 3; 
      %end;
      %if &FAG4 %then %do;
         if (&AgeGroup4) then NewAgeGroup = 4; 
      %end;
      %if &FAG5 %then %do;
         if (&AgeGroup5) then NewAgeGroup = 5; 
      %end;
   %end;
   %else %do;  
      NewAgeGroup = 1;
   %end;
run;

proc summary data=outlib.&OutDataName nway;
	class &TrtName;
	output out =armlevel;
run;
/* delete treatment level with only one sample; */
data delarm;
	set armlevel;
	if _FREQ_ <2 ;
run;
%let trtmissingflag =0
%Nobsvars(delarm);
%if &nobs > 0 %then %do;
	proc sort data=outlib.&OutDataName;
		by &TrtName;
	run;
	proc sort data=delarm;
		by &TrtName;
	run;
	data outlib.&OutDataName;
		merge outlib.&OutDataName delarm(in=ina);
		by &TrtName;
		if not ina;
	run;
	%let trtmissingflag=1;
	proc sql noprint;
		select &TrtName
		into :missingtrt separated by ','
		from delarm
	quit;
%end;
%Nobsvars(outlib.&OutDataName);
%if &nobs = 0 %then %do;
	%put ERROR: The number of records is insufficient for the analysis.;
	%let exiterror = 0;
	%goto exit;
%end;
data HROutDataName;
	set outlib.&OutDataName;
	if &TrtName ="&delarm" then delete;
	if %unquote(%LocaleGetAll(Progressive_Text, _case = u, _indexv = RSSTRESC, _indexvamp = 0)) or kindex(kupcase(RSSTRESC),"PD") then RSORRES2=RSSTRESC;
	else RSORRES2="";
run;
data outlib.&OutDataName;
	set outlib.&OutDataName;
	keep USUBJID &TrtName RSSTRESC ResponseTime censor AGE SEX RACE ETHNIC COUNTRY NewAgeGroup DCUTDAYS;
run;

* handling ScreenControlValue;
proc sort data=HROutDataName(keep=&TrtName) out=tmp nodupkey;
   by &TrtName;
run;
%nobsvars(tmp);
%let nTrtGroup = &nObs;
%put nTrtGroup=&nTrtGroup;
	data outlib.HROutDataName;
	set HROutDataName;
	run;
* check vartype for Treatment to see if numeric;
%let dsid=%sysfunc(open(HROutDataName));
%let TPos=%sysfunc(varnum(&dsid,&TrtName));
%global TType;
%let TType=%sysfunc(vartype(&dsid,&TPos)); 
%let rc1=%sysfunc(close(&dsid)); 

%if %length(&ScreenControlValue) %then %do;
   data _null_;
      set tmp end=_e_;
      name = "trt_" || trim(left(_n_));
      call symput(name,trim(left(&TrtName)));
      if _e_ then call symput("_nTrt_",_n_);
   run;
   
   %let ScreenControlValue = %AddQuotes(&ScreenControlValue);
   %let TRLFlag = 0;
   %do i = 1 %to &_nTrt_;
      %if %nrbquote("%upcase(&&trt_&i)") = %nrbquote(%upcase(&ScreenControlValue)) %then %do;
         %let TRLFlag = 1;
         %let ScreenControlValue = "&&trt_&i";
      %end;
   %end;
   %*put ScreenControlValue=&ScreenControlValue;
   %*put TRLFlag=&TRLFlag;

   %if ^&TRLFlag %then %do;
      %put [MS_S_0146];
      %put ERROR: The specified Treatment Control Level, &ScreenControlValue, was not found.;
      %goto exit;
   %end;
%end;
%else %do;
   * handling "Screen Failure" as the last level of ARM;
   %let _dSF_=0;
   data tmp1;
      set tmp end=_e_;
      if _e_ then do;
         if kupcase(&TrtName) = "SCREEN FAILURE" | kupcase(&TrtName) = "SCRNFAL" |
            kupcase(&TrtName) in (%unquote(%LocaleGetAll(ScreenFailure_Text, _case = u))) then do;
            call symput("_SF_",&TrtName);
            call symput("_nSF_",_n_);
            call symput("_dSF_",1);
            delete;
         end;
         else do;
            call symput("_dSF_",0);
         end;
      end;
   run;
   
   * Check if there is a level named "Placebo" or "Pbo";
   %let TRLFlag = 0;
   data _null_;
      %if &_dSF_ %then %do;
         set tmp1 end=_e_;
      %end;
      %else %do;
         set tmp end=_e_;     
      %end;
      if _e_ then call symput("LevelLast",trim(left(&TrtName)));
      if kupcase(&TrtName) = "PLACEBO" | kupcase(&TrtNAme) = "PBO" |
            kupcase(&TrtName) in (%unquote(%LocaleGetAll(Placebo_Text, _case = u))) then do;
         call symput("_ControlValue_",trim(left(&TrtName)));
         call symput("TRLFlag",1);
      end;
   run;
   %if ^&TRLFlag %then %do;
      %if &TType = N %then %let ScreenControlValue = &LevelLast;
      %else %let ScreenControlValue = "&LevelLast";
   %end;
   %else %do;
      %let ScreenControlValue = "&_ControlValue_";
   %end;
%end;
* re-ordering trtment levels;
data Trt_Num;
   set tmp;
   retain _c_ 0;
   if &TrtName = &ScreenControlValue then do;
      Trt_Num = &nTrtGroup;
   end;
   else do;
      _c_ + 1;
      Trt_Num = _c_;
   end;
   drop _c_;
run;
proc sort data=Trt_Num;
   by Trt_Num;
run;

%let armgroup=0;
data _null_;
   set Trt_Num;
   name = "ARM_" || trim(left(Trt_Num));
   call symput(name,&TrtName);
   call symput ("armgroup", trim(left(_n_)));
run;
%if &armgroup > 4 %then %do;
	%let RelativeSizeU = 67;
	%let RelativeSizeD = 33;
%end;
%else %do;
	%let RelativeSizeU = 85;
	%let RelativeSizeD = 15	;	
%end;

%if &nTrtGroup >1 %then %do;
	data HROutDataName;
	   if 0 then set Trt_Num;
	   if _n_ = 1 then do;
	      declare hash h(dataset: "work.Trt_Num", hashexp: 6);
	      h.defineKey("&TrtName");
	      h.defineData('Trt_Num');
	      h.defineDone();
	   end;
	   set HROutDataName;
	   if h.find() = 0;
	run;
	%let odata = &ProgressionSurPlots_OutData;
	%include "&MacroPath./DesignByVars.sas";
	%if &syserr > 4 or &exerr=1 %then %do;
	   %let exiterror = 1;
	 %end; 
	%if &exiterror %then %goto exit;

	proc phreg data=HROutDataName;
	   class Trt_Num(ref="&nTrtGroup");
	   model ResponseTime*censor(1) = Trt_Num;
	   hazardratio Trt_Num / diff=ref;
	   ods output parameterestimates=_params_type_ HazardRatios=Part3_1; 
	run;

	proc lifetest data=HROutDataName;
	   time ResponseTime*censor(1);
	   strata Trt_Num / test=logrank diff=Control("&nTrtGroup");
	   ods output Quartiles=Part2_1 HomTests=Part3_2 SurvDiff=Part3_21;
	run;

	proc summary data=HROutDataName completetypes noprint;
	   class Trt_Num;
	   class RSORRES2;
	   types Trt_Num*RSORRES2; 
	   output out=OS;
	run;

	* patch for missing;
	proc sort data=OS(keep=RSORRES2) out=tmp nodupkey;
	   by RSORRES2;
	run;

	%do i = 1 %to &nTrtGroup;
	   data tmp_&i;
	      set tmp;
	      Trt_Num = &i;
	      _FREQ_ = 0;
	   run;
   
	   %if &i = 1 %then %do;
	      data tmp_all;
	         set tmp_1;
	      run;
	   %end;
	   %else %do;
	      data tmp_all;
	         set tmp_all tmp_&i;
	      run;
	   %end;
	%end;
	data OS;
	   merge tmp_all OS;
	   by Trt_Num RSORRES2;
	run;

	data _null_;
	   set OS end=_e_;
	   by Trt_Num;
	   retain _c_ _cc_ 0;
	   _g_ = Trt_Num;
	   if first.Trt_Num then do;
	      _c_ = _FREQ_;     
	      if kupcase(RSORRES2) in (
	        %unquote(%LocaleGetAll(Progressive_Text, _case = u)),
	        %unquote("PD")) then _cc_ = 0;
	      else _cc_ = _FREQ_;
	   end;
	   else do;
	      _c_ + _FREQ_;
	      if kupcase(RSORRES2) not in (
	        %unquote(%LocaleGetAll(Progressive_Text, _case = u)),
	        %unquote("PD"))  then _cc_ + _FREQ_;
	   end;
	   if last.Trt_Num then do;
	      name = "Sum_" || trim(left(_g_));
	      call symputx(name,_c_);
	      name = "Sum_c_" || trim(left(_g_));
	      call symputx(name,_cc_);
	   end;
	run;
	* construct data with "censored" record;
	data OS_d OS_c;
	   set OS;
	   by Trt_Num;
	   if kupcase(RSORRES2) in (
	        %unquote(%LocaleGetAll(Progressive_Text, _case = u)),
	        %unquote("PD"))  then do;
	      output OS_d;
	   end;
	   else do;
	      output OS_c;
	   end;
	run;

	%nObsVars(OS_d);
	%if &nObs %then %do;
	   data OS_d;
	      set OS_d;
	      _O_ = 1;
	   run;
	%end;
	%else %do;
	   data OS_d;
	      length RSORRES2 $30;
	      do Trt_Num = 1 to &nTrtGroup;
	         RSORRES2 = "%nrbquote(%kupcase(%LocaleGet(Progressive_Text)))";
	         _FREQ_ = 0;
	         _O_ = 1;
	         output;
	      end;
	   run;
	%end;
	data OS_c; 
	   set OS_c;
	   by Trt_Num;
	   retain _O_;
	   if first.Trt_Num then _O_ = 3;
	   else _O_ = _O_ + 1;
	run;

	data OS_c2;
	   set OS;
	   by Trt_Num;
	   if first.Trt_Num;
	   _g_ = Trt_Num;
	   array _C_{&nTrtGroup} (%do i = 1 %to &nTrtGroup; &&Sum_c_&i %end;);
	   RSORRES2 = "%kupcase(%LocaleGet(Censored_Text))";
	   _FREQ_ = _C_(_g_);
	   _O_ = 2;
	run;
	data OS2;
	   set OS_d OS_c2 OS_c;
	   keep Trt_Num RSORRES2 _FREQ_ _O_;
	run;

	proc sort data=OS2;
	   by Trt_Num _O_;
	run;

	data OS3;
	   set OS2;
	   by Trt_Num;
	   /*make count and percent character variables so we can merge with
	    parts 2 and 3*/
	   length Count Percentage $20 ;
	
	   array _T_{&nTrtGroup} (%do i = 1 %to &nTrtGroup; &&Sum_&i %end;);
	   _g_ = Trt_Num;
	   Percentage = put(_FREQ_/_T_(_g_)*100,8.1);
	   Count = put(_FREQ_,8.0);
	   drop _T_:;
	run;
	data %do i = 1 %to &nTrtGroup; 
	      OS3_&i(rename=(Count=Count_&i Percentage=Percentage_&i))
	   %end;;
	   set OS3;
	   %do i = 1 %to &nTrtGroup;
	      if _g_ = &i then output OS3_&i;
	   %end;
	   drop _g_;
	run;
	data Part1;
	   merge %do i = 1 %to &nTrtGroup; OS3_&i %end;;
	   by _O_;
	   part = 1;
	   length col1 $50;
	   seq = _O_;
	   col1 = RSORRES2;
	   drop i _O_;      
	run;

	******************** PART 2 *************************;
	proc sort data=Part2_1; 
	   by Trt_Num percent;
	run;
	
	data Part2_2;
	   set Part2_1;
	   by Trt_Num;
	   length Count Percentage $20 LCL UCL $8;
	   array _T_{&nTrtGroup} (%do i = 1 %to &nTrtGroup; &&Sum_&i %end;);
	   _g_ = Trt_Num;
	   if LowerLimit=. then LCL = "NE";
	   else LCL = strip(put(LowerLimit,8.1));
	   if UpperLimit=. then UCL = "NE";
	   else UCL = strip(put(UpperLimit,8.1));
	   Percentage = '(' || trim(LCL) ||', '|| trim(UCL) ||')';
	   if estimate=. then Count = "NE";
	   else Count = trim(left(put(estimate,8.1)));
	   drop _T_:;
	run;
	data %do i = 1 %to &nTrtGroup; 
	      p2_&i(rename=(Count=Count_&i Percentage=Percentage_&i))
	   %end;;
	   set Part2_2;
	   %do i = 1 %to &nTrtGroup;
	      if Trt_Num = &i then output p2_&i;
	   %end;
	   drop _g_;
	run;
	data Part2;
	   merge  %do i = 1 %to &nTrtGroup; p2_&i %end;;
	   by percent;
	   length col1 $50;
	   part=2;
	   if percent=25 then do;
	      col1 = "%nrbquote(%LocaleGet(FirstQuartile_Text)) (95% CI)";
	      seq=1;
	   end;
	   else if percent=50 then do;
	      col1 = "%nrbquote(%LocaleGet(Median_Text)) (95% CI)";
	      seq=2;
	   end;
	   else if percent=75 then do;
	      col1 = "%nrbquote(%LocaleGet(ThirdQuartile_Text)) (95% CI)";
	      seq=3;
	   end;
	   * adjust empty p2_x dataset;
	   %do i = 1 %to &nTrtGroup;
	      if trim(left(Count_&i))="" then do;
	         Count_&i = "NE";
	         Percentage_&i = "(NE, NE)";
	      end;
	   %end;
	   drop STRATUM Trt_Num Estimate Transform LowerLimit UpperLimit LCL UCL;
	run;

	%let trAge = %nrbquote(%LocaleGet(Age_Text));
	%let trGender = %nrbquote(%LocaleGet(Gender_Text));
	%let trRace = %nrbquote(%LocaleGet(Race_Text));
	%let trHazardRatioforXSubgroup = %nrbquote(%LocaleGet(HazardRatioforXSubgroup_Text));
	%let trHazardRatioforGenderSG = %nrbquote(%LocaleFill(%nrbquote(&trHazardRatioforXSubgroup), %nrbquote(&trGender)));
	%let trHazardRatioforAgeSG = %nrbquote(%LocaleFill(%nrbquote(&trHazardRatioforXSubgroup), %nrbquote(&trAge)));
	%let trHazardRatioforRaceSG = %nrbquote(%LocaleFill(%nrbquote(&trHazardRatioforXSubgroup), %nrbquote(&trRace)));
	%let trHazardRatio = %nrbquote(%LocaleGet(HazardRatio_Text));
	%let trAgeGroup = %nrbquote(%LocaleGet(AgeGroup_Text));
	%let trLogranktest = %nrbquote(%LocaleGet(Logranktest_Text));
	%let trpvalue = %nrbquote(%LocaleGet(pvalue_Text));
	
	%if &nTrtGroup > 1 %then %do;
	   ******************** PART 3 *************************;
	   data Part3_1;
	      set part3_1 end=_e_;
	      length col1 $50 %do i = 1 %to %eval(&nTrtGroup-1); Count_&i Percentage_&i %end; $20;
	      retain %do i = 1 %to %eval(&nTrtGroup-1); Count_&i Percentage_&i HazardRatio_&i WaldLower_&i WaldUpper_&i %end;;
	      %do i = 1 %to %eval(&nTrtGroup-1);
	         if index(Description,"&i") then do;
	            Count_&i = strip(put(HazardRatio,8.3));
	            Percentage_&i = '(' || strip(put(WaldLower,8.3)) ||', '|| strip(put(WaldUpper,8.3)) ||')';
	            HazardRatio_&i = HazardRatio;
	            WaldLower_&i = WaldLower;
	            WaldUpper_&i = WaldUpper;
	         end;
	      %end;
	      * handling empty case;
	      %do i = 1 %to %eval(&nTrtGroup-1);
	         if trim(left(Count_&i)) = "" then do;
	            Count_&i = "NE";
	            Percentage_&i = "(NE, NE)";
	            HazardRatio_&i = .;
	            WaldLower_&i = .;
	            WaldUpper_&i = .;
	         end;
	      %end;
	      if _e_;
	      col1 = "&trHazardRatio (95% CI)";
	      part = 3;
	      seq = 1;
	      drop Description HazardRatio WaldLower WaldUpper;
	   run;

	   %if &nTrtGroup > 2 %then %do;
	      data Part3_21;
	         set part3_21 end=_e_;
	         length col1 $50 %do i = 1 %to %eval(&nTrtGroup-1); Count_&i %end; $20;
	         retain %do i = 1 %to %eval(&nTrtGroup-1); Count_&i %end;;
	         %do i = 1 %to %eval(&nTrtGroup-1);
	            if Stratum1 = &i then do;
	               Count_&i = strip(put(Dunnett,8.3));
	            end;
	         %end;
	         %do i = 1 %to %eval(&nTrtGroup-1);
	            if trim(left(Count_&i)) = "" then Count_&i = "NE";
	         %end;
	         if _e_;
	         col1 = "&trLogranktest (%nrbquote(%LocaleFill(%LocaleGet(pvaluecomparedtoX_Text), %nrbquote(%substr(%nrbquote(&ScreenControlValue),2,%length(&ScreenControlValue)-2)))))";
	         part = 3;
	         seq = 2;
	      run;
	      
	      data Part3_2;
	         set Part3_2;
	         length col1 $50 Count_1 $20;
	         col1 = "&trLogranktest (&trpvalue)";
	         part = 3;
	         seq = 3;
	         Count_1 = strip(put(ProbChiSq,8.3));
	      run;
	      
	      data Part3_2;
	         set Part3_21 Part3_2;
	      run;
	   %end;
	   %else %do;  
	      data Part3_2;
	         set Part3_2;
	         length col1 $50 Count_1 $20;
	         col1 = "&trLogranktest (&trpvalue)";
	         part = 3;
	         seq = 2;
	         Count_1 = strip(put(ProbChiSq,8.3));
	      run;
	   %end;
	
	   data part3;
	      set Part3_1(in=in1) Part3_2(in=in2);
	      keep col1 Count_: Percentage_: part seq;
	      drop Description HazardRatio: WaldLower: WaldUpper:;
	   run;
	
	%end;
	%else %do;
	   * one ARM level;
	   %let _DM_Flag_ = 0;
	   %let _LB_Flag_ = 0;
	   %let _CM_Flag_ = 0;
	   %let _MH_Flag_ = 0;
	%end;
	
	%let PFS_OutData_A1 = ;
	%PrepOutDS(PFS_OutData_A1,OutDataName_A1,progressionsurvival,_a1);
	data &PFS_OutData_A1;
	   set part1 part2 %if &nTrtGroup > 1 %then %do; part3 %end;;
	run;
	
	%let _OutputDatasets_ = &OutDataName..sas7bdat,&OutDataName_A1..sas7bdat;
%end;

* macros for sub-analysis;
%macro DM_SG_HR(_InDt,_gVar,_OutDt);
   %put Process for &_gVar in dataset &_InDt..;
   %let gVarType = ;
   %global HR_NoGo;
   %let HR_NoGo = 0;
   %VarAttributes(&_InDt,&_gVar,_Type=gVarType);
   %if &gVarType = N %then %let wStatement = %str(Where=( (&_gVar ^= .) & (ResponseTime ^= .) ));
   %else %let wStatement = %str(Where=( (&_gVar ^= " ") & (ResponseTime ^= .) ));

   * generate data for subgroup table;
   proc sort data=&_InDt(&wStatement) out=DM_SG;
      by &_gVar;
   run;

   * screen out data without reference level within each by-var to avoid ERROR;
   proc sort data=DM_SG(where=(&TrtName=&ScreenControlValue)) out=tmp(keep=&_gVar) nodupkey;
      by &_gVar;
   run;
   
   data DM_SG;
      if 0 then set tmp;
      if _n_ = 1 then do;
         declare hash h(dataset: "work.tmp", hashexp: 6);
         h.defineKey("&_gVar");
         h.defineDone();
      end;
      set DM_SG;
      if h.find() = 0;
   run;
   
   * screen out data with only control level;
   data DM_SG;
      set DM_SG;
      retain _c_ 0;
      by &_gVar;
      if first.&_gVar then _c_ = 1;
      else _c_+1;
      if last.&_gVar then do;
         if _c_ = 1 then delete;
      end;
   run;
   
   %nObsVars(DM_SG);
   %if ^&nObs %then %do;
      %let HR_NoGo = 1;
      %goto exit_DM_SG_HR;
   %end;

   proc phreg data=DM_SG ;
      class Trt_Num(ref="&nTrtGroup");
      by &_gVar;
      model ResponseTime*CENSOR(1) = Trt_Num;
      hazardratio Trt_Num / diff=ref;
      ods output HazardRatios=HR_12; 
   run;

   proc summary data=DM_SG completetypes noprint;
      *format DSDECOD $code.;
      class Trt_Num &_gVar CENSOR;
      types Trt_Num*&_gVar*CENSOR; 
      output out=HR_11;
   run;

   data %do i = 1 %to &nTrtGroup;
         DM_HR_1_&i
      %end;;
      set HR_11;
      %do i = 1 %to &nTrtGroup;
         if Trt_Num = &i then output DM_HR_1_&i;
      %end;
   run;

   %do i = 1 %to &nTrtGroup;
      %let j = %eval(&i*2);
      %let k = %eval(&j-1);
      data tmp1;
         set DM_HR_1_&i;
         if CENSOR = 0;
         rename _FREQ_ = Count_&k;
         drop Censor _TYPE_;
      run;
      proc summary data=DM_HR_1_&i completetypes noprint;
         class &_gVar;
         var _FREQ_;
         types &_gVar;
         output out=tmp2(drop=_TYPE_ _FREQ_) sum=Count_&j;
      run;
      data tmp3;
         merge tmp1 tmp2;
         by &_gVar;
      run;
      %if &i = 1 %then %do;
         data DM_HR_1;
            set tmp3;
            if Count_&j = . then Count_&j = 0;
            if Count_&k = . then Count_&k = 0;
         run;
      %end;
      %else %do;
         data DM_HR_1;
            merge DM_HR_1 tmp3;
            by &_gVar;
            if Count_&j = . then Count_&j = 0;
            if Count_&k = . then Count_&k = 0;
         run;
      %end;
   %end;
   
   data %do i = 1 %to %eval(&nTrtGroup-1); 
            HR_12_&i (rename=(HazardRatio=HazardRatio_&i WaldLower=WaldLower_&i WaldUpper=WaldUpper_&i))
        %end;;
      set HR_12;
      %do i = 1 %to %eval(&nTrtGroup-1); 
         if description = "Trt_Num &i vs &nTrtGroup" then output HR_12_&i;
      %end;
   run;
   
   data HR_12_All;
      merge %do i = 1 %to %eval(&nTrtGroup-1); HR_12_&i %end;;
      by &_gVar;
   run;

   %if %upcase(&_gVar) = NEWAGEGROUP %then %let _Sec_ = 1;
   %else %if %upcase(&_gVar) = SEX %then %do;
   	%let _Sec_ = 2;
   	%let _trgVar =%nrbquote(%LocaleGet(SEX_Text));
   %end;
   %else %if %upcase(&_gVar) = RACE %then %do;
   	%let _Sec_ = 3;
   	%let _trgVar =%nrbquote(%LocaleGet(Race_Text));
   %end;
   %else %if %upcase(&_gVar) = ETHNIC %then %do;
   	%let _Sec_ = 4;
   	%let _trgVar =%nrbquote(%LocaleGet(ETHNIC_Text));
   %end;
   %else %if %upcase(&_gVar) = COUNTRY %then %do;
   	%let _Sec_ = 5;
   	%let _trgVar =%nrbquote(%LocaleGet(Country_Text));
   %end;
   %else %if %upcase(&_gVar) = SITEID %then %do;
   	%let _Sec_ = 6;
   	%let _trgVar =%nrbquote(%LocaleGet(SITEID_Text));
   %end;
   %else %let _sec_ = 1; * one section case;

   data &_OutDt;
      length DM_Subgroup $32 %do i = 1 %to %eval(&nTrtGroup-1); HR_&i HR_LCL_&i HR_UCL_&i %end; $10;
      merge DM_HR_1 HR_12_All;
      by &_gVar;
      %if %upcase(&_gVar) = NEWAGEGROUP %then %do;
         %do i = 1 %to 5;
            if NewAgeGroup = &i then DM_Subgroup = "&&AgeGroup&i";
         %end;
      %end;
      %else %do;
         DM_Subgroup = "&_trgVar="||trim(left(&_gVar));
      %end;
      _seq_ = _n_;
      _sec_ = &_Sec_;
      %let l = %eval(&nTrtGroup*2-1);
      %do i = 1 %to %eval(&nTrtGroup-1);
         %let k = %eval(&i*2-1);
         if (Count_&k = 0) | (Count_&k = .) | (Count_&l = 0) | (Count_&l = .) then do;
            HR_&i = "NE";
            HR_LCL_&i = "(NE,";
            HR_UCL_&i = "NE)";
         end;
         else do;
            if HazardRatio_&i = . then HR_&i = "NE";
            else HR_&i = strip(put(HazardRatio_&i,8.2));
            if WaldLower_&i = .  then HR_LCL_&i = "(NE,";
            else HR_LCL_&i = "(" || strip(put(WaldLower_&i,8.2)) || ",";
            if WaldUpper_&i = . then HR_UCL_&i = "NE)";
            else HR_UCL_&i = strip(put(WaldUpper_&i,8.2)) || ")";
         end;
         HR_CL_&i = trim(left(HR_LCL_&i)) || " " || trim(left(HR_UCL_&i));
      %end;
      drop Trt_Num Description &_gVar;
      *label HazardRatio="&trHazardRatio" WaldLower="Lower CI" WaldUpper="Upper CI";
   run;
   %exit_DM_SG_HR:; 
   %put HR_NoGo = &HR_NoGo;
%mend DM_SG_HR;

*** Analysis: Hazard Ratio Estimates for Subgroup of Demographics;
%if &_DM_Flag_ & &nTrtGroup >1 %then %do;

	* for Age subgroup;
   %if %upcase(&TrtName) ne AGE %then %do;
      %DM_SG_HR(HROutDataName,NewAgeGroup,DM_HR_Age);
      %let HR_NoGo_A2_1 = &HR_NoGo;
   %end;
   %else %do;
      %let HR_NoGo_A2_1 = 1;
   %end;	
   
   * for Gender subgroup;
   %if %upcase(&TrtName) ne SEX %then %do;
      %DM_SG_HR(HROutDataName,SEX,DM_HR_Sex);
      %let HR_NoGo_A2_2 = &HR_NoGo;
   %end;
   %else %do;
      %let HR_NoGo_A2_2 = 1;
   %end;
   
   * for Race subgroup;
   %if %upcase(&TrtName) ne RACE %then %do;
   	%let HR_NoGo_A2_3 = 0;
   	%if %sysfunc(findw(&ADSLVARS, RACE)) %then %do;
			%DM_SG_HR(HROutDataName,RACE,DM_HR_Race);
			%let HR_NoGo_A2_3 = &HR_NoGo;
		%end;
		%else %let HR_NoGo_A2_3 = 1;
   %end;
   %else %do;
      %let HR_NoGo_A2_3 = 1;
   %end;
   
   * for Ethnic subgroup;
   %if %upcase(&TrtName) ne ETHNIC %then %do;
   	%let HR_NoGo_A2_4 = 0;
   	%if %sysfunc(findw(&ADSLVARS, ETHNIC)) %then %do;
			%DM_SG_HR(HROutDataName,Ethnic,DM_HR_Ethnic);
			%let HR_NoGo_A2_4 = &HR_NoGo;
		%end;
		%else %let HR_NoGo_A2_4 = 1;
   %end;
   %else %do;
      %let HR_NoGo_A2_4 = 1;
   %end;
   
   * for country subgroup;
   %if %upcase(&TrtName) ne COUNTRY %then %do;
   	%let HR_NoGo_A2_5 = 0;
   	%if %sysfunc(findw(&ADSLVARS, COUNTRY)) %then %do;
			%DM_SG_HR(HROutDataName,COUNTRY,DM_HR_Country);
			%let HR_NoGo_A2_5 = &HR_NoGo;
      %end;
      %else %let HR_NoGo_A2_5 = 1;
   %end;
   %else %do;
      %let HR_NoGo_A2_5 = 1;
   %end;
   
   %if ^&HR_NoGo_A2_1 | ^&HR_NoGo_A2_2 | ^&HR_NoGo_A2_3 |
   ^&HR_NoGo_A2_4 | ^&HR_NoGo_A2_5 %then %let _Skip_A2 = 0;
   %else %let _Skip_A2 = 1;
  %put  HR_NoGo_A2_1=&HR_NoGo_A2_1 HR_NoGo_A2_2=&HR_NoGo_A2_2 HR_NoGo_A2_3=&HR_NoGo_A2_3 HR_NoGo_A2_4=&HR_NoGo_A2_4 HR_NoGo_A2_5=&HR_NoGo_A2_5 _Skip_A2=&_Skip_A2;
   %if ^&_Skip_A2 %then %do;
		data DM_HR_All;
         set 
            %if ^&HR_NoGo_A2_1 %then DM_HR_Age;
            %if ^&HR_NoGo_A2_2 %then DM_HR_Sex;
            %if ^&HR_NoGo_A2_3 %then DM_HR_Race;
            %if ^&HR_NoGo_A2_4 %then DM_HR_Ethnic;
            %if ^&HR_NoGo_A2_5 %then DM_HR_Country;
        /*    %if ^&HR_NoGo_A2_6 %then DM_HR_Site;*/
            ;
      run;
      %do i = 1 %to 5;
         %let PFS_OutData_A2_P&i = ;
         %PrepOutDS(PFS_OutData_A2_P&i,OutDataName_A2_P&i,overallsurvival,_a2&i);
      %end;
      
      %let PFS_OutData_A2_all= ;
      %PrepOutDS(PFS_OutData_A2_all,OutDataName_DM_all,overallsurvival,_DMAll);
      data &PFS_OutData_A2_all;
      	set DM_HR_ALL;
      	if DM_Subgroup = "Missing" then delete;
     * 	if WaldUpper_1 =. or WaldLower_1=. then delete;
      run;
      proc sort data =&PFS_OutData_A2_all;
      	by DM_Subgroup;
      run;
      /* output each subgroup for forest plot;*/
      data &PFS_OutData_A2_P1 &PFS_OutData_A2_P2 &PFS_OutData_A2_P3 &PFS_OutData_A2_P4 &PFS_OutData_A2_P5;
      	set DM_HR_All;
      	if _sec_ = 1 then output &PFS_OutData_A2_P1;
         else if _sec_ = 2 then output &PFS_OutData_A2_P2;
         else if _sec_ = 3 then output &PFS_OutData_A2_P3;
         else if _sec_ = 4 then output &PFS_OutData_A2_P4;
         else if _sec_ = 5 then output &PFS_OutData_A2_P5;
      run;
      %do i = 1 %to 5;  
         %if %sysfunc(exist(&&PFS_OutData_A2_P&i)) %then %do;
            %let _OutputDatasets_ = &_OutputDatasets_,&&OutDataName_A2_P&i...sas7bdat;
         %end;
      %end;      
      
   %end;
%end;

%global _mainjslfile_ FullAPName;
%let _mainjslfile_ = &ProgressionPlotsScript;
%let FullAPName=%str(Progression Free Survival);

* put JSL statements;
data _null_;
   file "&ProgressionPlotsScript" %if %nrbquote(&SYSENCODING) ne wlatin1 %then encoding="utf-8";;
   put "//!";
   put "Names Default to Here(1);";
   put "objRefNS = newNamespace();";
   put "jg:currentObjRefNamespaceName = objRefNS << getName;";
   put "objRefNS:DataList = {};";
   put "objRefNS:DataNameList = {};";
   put "objRefNS:TabButtonList = {};";
   put "objRefNS:TabCodeList = {};";
   put "objRefNS:TabAfterCodeList = {};";
   put "objRefNS:TabOpenList = {};";
   put "objRefNS:CreateReportList = {};";
   put "objRefNS:CreateReportListone={};";
   put "objRefNS:CreateReportListtwo= {};";
   put "objRefNS:NoteDomain = ""RS"";";
   put "objRefNS:ARMLabel = ""&ARMlabel"";";
   put "objRefNS:ProgressionLabel = ""&ProgressionLabel"";";
   put "objRefNS:StudyName = ""&StudyName"";";
   put "objRefNS:ClientOutPath = ""&ClientOutPath"";";
   %if %UPCase(&TrtUse) ne NONE  %then %do;
   	put "objRefNS:armgroup = &armgroup;";
   %end;
   %else %do;
   	put "objRefNS:armgroup = 1;";
   %end;
	put "objRefNS:RelativeSizeU = &RelativeSizeU;";
	put "objRefNS:RelativeSizeD = &RelativeSizeD;";
	put "objRefNS:shadedCI = &shadedCI;";
	put "objRefNS:TrtUse = ""&TrtUse"";";

   put "objRefNS:openDTExpr = Expr(";
   put "   If(IsEmpty(objRefNS:rsd),";
   put "objRefNS:rsd=open(""&ClientOutPath.&OutDataName..sas7bdat"",invisible);";
	put "      jg:ApplyValueOrderToTables(EvalList({objRefNS:rsd}), ""&StudyName"");";
	put "      reportRefNS:linkAnalysisTableToADSLTable(objRefNS:rsd);";
   %if %UPCase(&TrtUse) ne NONE  %then %do;
   put "//      objRefNS:rsd << Color By Column(Column(""&ARMlabel""));";
   %end;
   put "    ,"; /*else*/
   put " Current Data Table(objRefNS:rsd);"; 
   put "   );";
   put ");";
   put "  objRefNS:openDTExpr;";
   
   put "objRefNS:SPfunc = Function({},{default local},";
  
   put "  objRefNS:openDTExpr;";
   
   put "   VlistBox(";
	%if &AnalysisType = PFS %then %do;
			put "		TextBox(locale:get(""AnalysisTypePFSProgInclBothProgDiseaseAndDeath_Text""),<<Set Wrap(800),<<Set Font Size(9)),";
	%end;
	%if &AnalysisType = TTP %then %do;
			put "		TextBox(locale:get(""AnalysisTypeTTPProgOnlyInclProgDisease_Text""),<<Set Wrap(800),<<Set Font Size(9)),";
	%end;
   put "		TextBox(locale:fill(locale:get(""NoteSubjectsWereCensoredUsingX_Text""), {""&CensorDateValue""}),<<Set Wrap(500),<<Set Font Size(9)),";
	%if &DCUTDmissflag %then %do;
		put "		TextBox(locale:fill(locale:get(""NoteXSubjectsWereRemFromAnDueToMissCensDateInfo_Text""), {""&missingObs""}),<<Set Wrap(800),<<Set Font Size(9)),";
	%end;
   put "    HlistBox(";
   put " objRefNS:gb = Survival(";
   put "			Y(:Name(""&ProgressionLabel"")),";
   put "			Censor( :censor),";
   %if %UPCase(&TrtUse) ne NONE  %then %do;
   put "			Grouping( :Name(""&ARMlabel"")),";
   %end;
   /*%if %length(&SplitResults) %then %do;
   put "       by(:Name(""&DesignByVars"")),";
      %do j=1 %to &ndbv;
         put "       by(Column(""&&dbv&j"")),";
      %end;
   %end;*/
	put "		//	Competing Causes(:Name(""&ARMlabel"") ),";
   put "			Weibull Lines( 1 ),";
	put "			Hazard Plot( 1 ),";
	put "			Failure Plot( 0 ),";
	put "			Automatic Recalc( 1 ), Survival Plot( 1 ),";
	put "			Show Points( 1 ),";
	put "			Show Shaded Simultaneous CI( &shadedCI ),";
	put "			SendToReport(";
	put "				Dispatch(";
	put "					{""Survival Plot""},";
	put "					""2"",";
	put "					ScaleBox,";
	put "					{Min( -0.05 ), Max( 1.05 ), Inc( 0.2 ), Minor Ticks( 1 )}";
	put "				),";
	put "				Dispatch(";
	put "					{""Survival Plot""},";
	put "					""Surv Plot"",";
	put "					FrameBox,";
	put "					{DispatchSeg( Marker Seg( 1 ), {Marker( ""&CensorMark"" )} )}";
	put "				)";
	put "			)";
   put "      ),";/*end of survival;*/
 
 	put "include(""$LS_HOME\JSLFiles\HazardRatioPlot.JSL"");";
	%if &_DM_Flag_ & %UPCase(&TrtUse) ne NONE & &nTrtGroup >1  %then %do;

	      put "   if(IsEmpty(objRefNS:A2_all),";
	      put "      objRefNS:A2_all = open(""&ClientOutPath.&OutDataName_DM_all..sas7bdat"",Use Labels For Var Names( 0 ),invisible);";
	      put "   );";
	      
	      put " Outlinebox(locale:get(""HazardRatioForestPlots_Text""),";
	      %if &nTrtGroup = 2 %then %do;
	      put "   objRefNS:HR_plot = vListBox(";
	      put "      TextBox(""   "" || locale:fill(locale:get(""XvsX_Text""), {""%trim(%nrbquote(&ARM_1))"", ""%trim(%nrbquote(&&ARM_&nTrtGroup))""}));";
	      put "      ,";
	      put "			HListBox(";
	      put "         objRefNS:HR_ForestPlot(objRefNS:A2_all,""HazardRatio_1"",""WaldLower_1"",""WaldUpper_1"",""DM_Subgroup"",";
	      put "            locale:fill(locale:get(""HazardRatioforXSubgroup_Text""), {locale:get(""Demographics_Text"")}),""&trHazardRatio"",""&trDemographics"",""objRefNS:DM_P3"");";
	      put "   );";
	      put "   );"; /* end of vListBox */
	      %end;
	      %else %do;
	      put "   objRefNS:HR_plot = vListBox(";
	      put "			VListBox(";
	      %do i = 1 %to %eval(&nTrtGroup-1);
	      put "      TextBox(""   "" || locale:fill(locale:get(""XvsX_Text""), {""%trim(%nrbquote(&&ARM_&i))"", ""%trim(%nrbquote(&&ARM_&nTrtGroup))""}));";
	      put "      ,";
	      put "		//	VListBox(";
	     	      
	      put "         objRefNS:HR_ForestPlot(objRefNS:A2_all,""HazardRatio_&i"",""WaldLower_&i"",""WaldUpper_&i"",""DM_Subgroup"",";
	      put "            locale:fill(locale:get(""HazardRatioforXSubgroup_Text""), {locale:get(""Demographics_Text"")}),""&trHazardRatio"",""&trDemographics"",""objRefNS:DM_P3""),";
	      put "  // );";
	      %end;	      
	      put "   );";
	      put "   );"; /* end of vListBox */
	      %end;
	      put "   );";
	%end;
	      
     /* end of Hazard Ratio function*/
   %if &stageFlag %then %do;
   put "       ColumnSwitcherObj =objRefNS:gb << Column Switcher(";
   put "       ""&ARMlabel"",{""&ARMlabel"",:stage}";
   put "       ),";
   %end;
   put "      );"; /* end of VListbox;*/
   put "      );"; /* end of VListbox;*/
   put "      );"; /* end of function SPfunc;*/
   
   put 'InsertInto(objRefNS:DataList,{{"objRefNS:rsd","objRefNS:A2_all"}});';
   put "InsertInto(objRefNS:DataNameList,{{locale:get(""OverallSurvival_Text""),locale:fill(locale:get(""HazardRatioforXSubgroups_Text""), {locale:get(""Demographics_Text"")})}});";
   put "InsertInto(objRefNS:TabButtonList, expr(locale:get(""ProgressionFreeSurvival_Text"")));";
   put 'InsertInto(objRefNS:TabCodeList,expr(objRefNS:SPfunc()));';
   put 'InsertInto(objRefNS:TabAfterCodeList, " ");';
   put 'InsertInto(objRefNS:TabOpenList,1);';
   put "InsertInto(objRefNS:CreateReportList,{{objRefNS:gb,objRefNS:HR_plot}});";  
   put " ";
   
   %SubjectDrillDowns; 
      
	put "if(hostis(windows), _slash_ = ""\"", _slash_ = ""/"");";
	put "if(Host Is(Windows),";
	put "   JMPClinicalDir = ConvertFilePath(""$CLINICAL_HOME\"", absolute, windows),";
	put "   JMPClinicalDir = ConvertFilePath(""$CLINICAL_HOME/"", absolute, POSIX)";
	put ");";     
	put "include( JMPClinicalDir||""JSLFiles/PatientsAtRisk.jsl"");";
	
   put "InsertInto(objRefNS:ActionButtonList[1], {expr(locale:get(""ShowPatientsAtRisk_Text""))});";
   put "InsertInto(objRefNS:ActionCodeList[1],Expr(objRefNS:PatientsAtRisk()));";
   put "InsertInto(objRefNS:ActionSelectionCountList[1],0);";
   put "InsertInto(objRefNS:ActionRequirementFlagList[1],0);";
   put "InsertInto(objRefNS:ActionIconList[1],""/install/lifesciences/documentation/icons/clinical/PatientsAtRisk.gif"");";
   put "objRefNS:ActionDataTableList = {{""objRefNS:rsd""}};";
   /*
	put "InsertInto(objRefNS:ActionButtonDesc, {Expr(locale:get(""ClicktoView_Text""))});";
	put "objRefNS:ActionButtonList={expr(locale:get(""GraphTimeTrends_Text""))};";
	put "objRefNS:ActionIconList = {""/install/lifesciences/documentation/icons/clinical/LinesPlot.gif""};";
	put "objRefNS:ActionCodeList ={objRefNS:PatientsAtRisk()};";
      put "objRefNS:ActionDataTableList = {{""objRefNS:rsd""}};";
      put "objRefNS:ActionSelectionCountList = {1};";
      put "objRefNS:ActionRequirementFlagList = {0};";  */  
*/
run;

%PackageInsertFile(jsl code from &ProcessName, &OutPath, &ProgressionPlotsScriptName, &OutDataName..sas7bdat, yes);


%exit:;
%ExitSDSProcess(&OutPath, &ProcessName, &syserr, &exiterror, &clockrunning);

%mend ProgressionFreeSurvivalPlots;

* run the macro;
%ProgressionFreeSurvivalPlots;


