/*
JMP Genomics Analytical Process

Name: Hierarchical Clustering
Author: Russ Wolfinger, SAS Institute Inc.
Support: Tzu-Ming Chu, SAS Institute Inc.
History:
   sasrdw 22Jan04 initial coding 
	tzuchu 30Aug07 added color heat map for compare vars 

Description: 
Hierarchical Clustering creates a
tree of relationships of the observations
(rows) of a SAS data set using one of 
several methods.  The variables themselves
may also be clustered to create a two-way 
organization. All methods are based on an 
agglomerative hierarchical clustering 
procedure. Each observation begins in a 
cluster by itself. The two closest clusters
are merged to form a new cluster that 
replaces the two old clusters. Merging of 
the two closest clusters is repeated 
until only one cluster is left. The 
various clustering methods differ in 
how the distance between two clusters 
is computed.
*/

/*
-----------------------------------------------------------
Copyright (C) 2004 SAS Institute, Inc. All rights reserved. 

Notice: 
The above copyright notice and this notice must appear in
any whole or partial copy of this code and any related
documentation. Permission to use, copy, and modify the
source code contained in this file is hereby granted, but
is limited to customers of SAS with a valid license for the
SAS software product with which this file was provided. 
Except as expressly set forth herein, the terms of such
license apply.
-----------------------------------------------------------
*/

*ProcessVariables;

%global StudyName InData Variables HierarchicalClusteringMethod TwoWayClustering CenterRows ScaleRows Theme
Standardize ImputeMissing OrderingVar LabelVar CompareVars ByVars ListVars WhereStmt OutPath ServerOutPath MacroPath 
HierarchicalClusteringScript HierarchicalClustering_OutData;

*ProcessBody;

%macro HierarchicalClustering;

*initialization and argument checking;
%let ProcessName = HierarchicalClustering; 

%include "&MacroPath./UtilityMacros.sas" / nosource;

%PathName(&InData);
%if &exiterror %then %do;
   %put ERROR: Input dataset specification is invalid.;
   %goto exit;
%end;
%let InData = &TmpName;
libname InLib "&TmpPath";
libname OutLib "&OutPath";

%if %length(&Variables &ListVars)=0 %then %do;
	%put ERROR: You must specify variables whose rows are to be clustered.;
	%let exiterror=1;
	%goto exit;
%end;
%if %length(&OrderingVar) %then %do;
	%let ovtype =;
	%VarAttributes(InLib.&InData,&OrderingVar,_type=ovtype);   
   %if &ovtype ^= N %then %do;
   	%put ERROR: Ordering variable must be numeric.;
   	%let exiterror = 1;
   	%goto exit;
   %end;
%end;
%if %length(&ListVars) %then %do;
	%if ^%index(&ListVars,:) & ^%index(&ListVars,-) & ^%index(&ListVars,_numeric_) & ^%index(&ListVars,_character_) & ^%index(&ListVars,_all_) %then %do;
		%put ERROR: You did not specify the List variables correctly, please use valid symbols based on the question mark next to it.;
		%let exiterror = 1;
   	%goto exit;
	%end;
%end;


%PrepOutDS(HierarchicalClustering_OutData,OutDataName,&InData,_hc);

%if %length(&WhereStmt) %then %do;
	data ind;
		set InLib.&InData;
		where &WhereStmt;
	run;
	%let indat = ind;
%end;
%else %do;
	%let indat = InLib.&InData;
%end;

* center and/or scale rows;
%if &CenterRows=Yes or &ScaleRows=Yes %then %do;
	data ind;
		set &indat;
		
		%if &CenterRows=Yes %then %do;
			_rowmean_ = mean(of &Variables &ListVars);
		%end;
		%else %do;
			_rowmean_ = 0;
		%end;

		%if &ScaleRows=Yes %then %do;
			_rowstd_ = std(of &Variables &ListVars);
		%end;
		%else %do;
			_rowstd_ = 1;
		%end;
		
		array v{*} &Variables &ListVars;
		do i=1 to dim(v);
			if _rowstd_ > 0 then v[i] = (v[i] - _rowmean_)/_rowstd_;
			else v[i] = .;
		end;
		drop i;
	run;	
	%let indat = ind;		
%end;	

data &HierarchicalClustering_OutData;
	set &indat;
run;

/*---create variable list---*/
%if ^%length(&Variables) and ^%length(&ListVars) %then %do;
	%put ERROR: You must specify Variables Whose Rows are to be Clustered.;
	%let exiterror = 1;
	%goto exit;
%end;

* use a data set to save variable names and labels;
* this is quicker than defining macro variables for really wide data sets and avoids string truncation issues;
data dsix;
   set &HierarchicalClustering_OutData;
   keep &Variables &ListVars;
   if (0);
run;
data outlib.hcvars;
	set dsix;
run;

%let _dsix_  = %sysfunc(open(dsix));
%let nix  = %sysfunc(attrn(&_dsix_, NVARS));

/*
proc transpose data=a out=at;
   var &Variables &ListVars;
run;
%if &syserr %then %goto exit;

data _null_;
   set at end=_last;
   length ix$ 16;
   ix = "ix" || trim(left(_n_));
   call symput(ix,trim(left(_name_)));
   if (_last) then call symput("nix",trim(left(_n_)));
run;
%if &syserr %then %goto exit;
*/

* create JSL file;
%PrepOutFile(HierarchicalClusteringScript,HierarchicalClusteringScriptName,&ProcessName Script for &OutDataName..jsl);

* convert names to labels for JMP;
%let LabelVar0 = &LabelVar;
%if %length(&LabelVar) %then %do;
   %CheckLabel(&HierarchicalClustering_OutData,LabelVar);
%end;
%if %length(&OrderingVar) %then %do;
   %CheckLabel(&HierarchicalClustering_OutData,OrderingVar);
%end;
/*
%do i=1 %to &nix;
	%CheckLabel(&HierarchicalClustering_OutData,ix&i);
%end;
*/
%if %length(&CompareVars) %then %do;
	%let i = 1;
	%do %while (%length(%scan(&CompareVars,&i,' ')));
		%let cvi = %scan(&CompareVars,&i,' ');
		%CheckLabel(&HierarchicalClustering_OutData,cvi);	
		%let cv&i = &cvi;
		%let i=%eval(&i+1);
	%end;
	%let ncv = %eval(&i-1);
%end;
%if %length(&ByVars) %then %do;
	%let i = 1;
	%do %while (%length(%scan(&ByVars,&i,' ')));
		%let bvi = %scan(&ByVars,&i,' ');
		%let bv&i = &bvi;
		%CheckLabel(&HierarchicalClustering_OutData,bvi);	
		%let byv&i = &bvi;
		%let i=%eval(&i+1);
	%end;
	%let nbyv = %eval(&i-1);
%end;

* subset dataset to avoid By Variable issue in JMP; 
%if %length(&ByVars) %then %do; 
	* create JSL file;
	%let HierarchicalClusteringJournal = ;
	%PrepOutFile(HierarchicalClusteringJournal,HierarchicalClusteringJrnName,&ProcessName Journal for &OutDataName..jrn);

	%SliceData(&HierarchicalClustering_OutData,&ByVars,LevelData,&OutPath,_SubDataList=sList);
	* set button names;
	data _null_;
		set LevelData end=_e_;
		%do v = 1 %to &nbyv;
			name&v = "bvLevel&v._" || trim(left(_n_));
			call symput(name&v,&&bv&v);
		%end;
		if _e_ then call symput("nLevel",_n_);
	run;
	%let ButtontList = ;
	%do l = 1 %to &nLevel;
		%do v = 1 %to &nbyv;
			%if &v = 1 %then %let tmp	= &&bv&v=&&bvLevel&v._&l;
			%else %let tmp = &tmp & &&bv&v=&&bvLevel&v._&l; 
		%end;
		%let ButtontList = &ButtontList &tmp;
		%if &l ne &nLevel %then %let ButtontList = &ButtontList |;
	%end;	 
	%*put &HierarchicalClusteringJrnName;
	%*put &ButtontList;
	%*put &HierarchicalClusteringScriptName;
	%*put &sList;
	%Journal(&OutPath.&HierarchicalClusteringJrnName,Hierarchical Clustering,
		%nrbquote(&ButtontList),&HierarchicalClusteringScriptName,_nWithinDs=1,
		_SubDataList=&sList);
%end;

* check for _NAME_ variable for partial correlation drill down;
%let _ds_ = %sysfunc(open(&HierarchicalClustering_OutData));
%let _found_ = %sysfunc(varnum(&_ds_,_NAME_));
%let _rc_=%sysfunc(close(&_ds_));
%if &_found_ %then %let NameVar = _NAME_;
%else %if %length(&LabelVar) %then %let NameVar = &LabelVar;
%else %let NameVar = ;

* create JSL file;
%global _mainjslfile_ FullAPName;
%let _mainjslfile_ = &HierarchicalClusteringScript;
%let FullAPName=%str(Hierarchical Clustering);

* put JSL statements; 
data _null_;
   file "&HierarchicalClusteringScript" encoding="utf-8";
	put "//!";
	put "Names Default to Here(1);";
	put "DataList = {};";
	put "TabButtonList = {};";
	put "TabCodeList = {};";
	put "TabOpenList = {};";
	put "Here:CreateReportList = {};";
	put "vb = Function({},{default local},";
	put "   If(IsEmpty(Here:hcdata),";
		%if %length(&ByVars) %then %do;
			put "      Here:hcdata = open(""&ClientOutPath""||jg:outds1,invisible);";
			%let ByVars = ;
			%let ByVarsFlag = 1;
		%end;
		%else %do;
			put "      Here:hcdata = open(""&ClientOutPath.&OutDataName..sas7bdat"",invisible);";
			%let ByVarsFlag = 0;
		%end;	
		%if %length(&LabelVar) %then %do;
			put "      %str(try(column(%"&LabelVar%") << label(1));)";
		%end;
	put "   );";
	put "   Current Data Table(Here:hcdata);";
	put "   HListBox(";
   put "   Here:HCWin = Hierarchical Cluster(Y(";
	%do i=1 %to &nix;
		%let v = %sysfunc(varlabel(&_dsix_,&i));
		%if ^%length(&v) %then %let v = %sysfunc(varname(&_dsix_,&i));
		put " Name(""&v"")" @@;
		%if (&i<&nix) %then %do;
			put "," @@;
		%end;
		%else %do;
			put "),";
		%end;
	%end;
   put "   Method(&HierarchicalClusteringMethod), Dendrogram Scale( Distance Scale ), " /
   %if %length(&ByVars) %then %do;
   	"      By(" / 
   	%do i=1 %to %eval(&nbyv-1);
   		":&&byv&i," /
   	%end;
   	":&&byv&nbyv)," /        
	%end;
   %if %length(&OrderingVar) %then %do;
   	"      Ordering(column(""&OrderingVar""))," /       
	%end;
   %if %upcase(&Standardize) ^= YES %then %do;
      "      Standardize(0)," /
   %end;
   %if %upcase(&ImputeMissing) = YES %then %do;
   	"		 Missing Value Imputation(1),"/
   %end;
	%if %length(&CompareVars) %then %do;
		"      More Color Map Columns( "
		%do i=1 %to %eval(&ncv-1);
			"column(""&&cv&i""),"
		%end;
		"column(""&&cv&i"") )," /
	%end;

   "      Number of Clusters(10), Color Clusters(1), Color Map(&Theme), Save Clusters(1)," /
   "      Legend(1), Save Display Order(1)," /
	%if &ByVarsFlag %then %do;
		"      SendToReport(" /
		"         Dispatch("/
		"            {},"/
		'            "Hierarchical Clustering",'/
		"            OutlineBox,"/
		'            Set Title( "Hierarchical Clustering  (" || jg:subTitle || ")")'
		"         )"/
		"      ),"/
	%end;
	");";
	%if %upcase(&TwoWayClustering) = YES %then %do;
		put "Here:HCWin<<Two Way clustering;";
	%end;
	%if %length(&LabelVar) %then %do;
		put "Here:Filter = Here:hcdata << Data Filter(";
		put "   Add Filter(";
		put "     try( Columns(%str(column(%"&LabelVar%"))))";
		put "   )";
		put ");";
	%end;
	put "));";
	put "   pdcFunc =  Function({},{},";
	put "      outpath = ""&OutPath"";";
	put "      ::serveroutpath = ""&ServerOutPath"";";
	put "      ::clientoutpath = ""&ClientOutPath"";";
	put "      namevar = ""&NameVar"";";
	put "      labelvar = ""&LabelVar0"";";
	put "      hcvars = ""hcvars"";";
	put "      include( ""$LS_HOME/Generator/DialogHelperFunctions.jsl"");";
	put "      include( ""$LS_HOME/JSLFiles/Hierarchical Clustering Drill Down Expressions.jsl"");";
	put "      HCSaveSubset;";
	put "      HCTranspose;";
	put "      LaunchPartialCorrelationDiagram;";
	put "      jg:runProcess(1,""PartialCorrelationDiagram"");";
	put "   );";		

	put "InsertInto(DataList,{{""Here:hcdata""}});";
	put "InsertInto(TabButtonList, Expr(locale:get(""HierarchicalClustering_Text"")));";
	put "InsertInto(TabCodeList,expr(vb()));";
	put "InsertInto(TabOpenList,1);";
	put "InsertInto(CreateReportList,{{Here:HCWin}});";
	
	put "ActionButtonList= {{Expr(locale:get(""PartialCorrelationDiagram_Text""))}};";
	put "ActionCodeList= {{pdcFunc()}};";
	put "ActionButtonDesc = {Expr(locale:get(""SelectaSmallClusterthenClickButton_Text""))};";		
run;

%let _dsix_ = %sysfunc(close(&_dsix_));

%if &ByVarsFlag %then %do;
	%PackageInsertFile(jsl code from &ProcessName, &OutPath, &HierarchicalClusteringJrnName, "&OutDataName..sas7bdat,hcvars.sas7bdat", text/plain, yes);
	%let sList2 = %scan(&sList,1).sas7bdat;
	%let i = 2;
	%do %while(%length(%scan(&sList,&i)));
		%let sList2 = &sList2 , %scan(&sList,&i).sas7bdat;
		%let i = %eval(&i+1);
	%end;
	%PackageInsertFile(jsl code from &ProcessName, &OutPath, &HierarchicalClusteringScriptName, "&sList2", text/plain, Yes, No);
%end;
%else %do; 
	%PackageInsertFile(jsl code from &ProcessName, &OutPath, &HierarchicalClusteringScriptName, "&OutDataName..sas7bdat,hcvars.sas7bdat", text/plain, yes);
%end;

%exit:;
%ExitSDSProcess(&OutPath, &ProcessName, &syserr, &exiterror, &clockrunning);
quit;
%mend HierarchicalClustering;

* run the macro;
%HierarchicalClustering;


